/*
 * Copyright (C) 2003 Andrea Bergia <andreabergia@yahoo.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License , or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to
 *
 *   The Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111, USA.
 */

#include <kparts/part.h>
#include <klibloader.h>
#include <kdebug.h>
#include <klocale.h>
#include <knotifyclient.h>
#include <qvaluelist.h>

#include <kurl.h>
#include <qfile.h>
#include <qtextstream.h>
#include <kio/netaccess.h>

#include <qlayout.h>
#include <kdialog.h>
#include <klistview.h>

#include <kdevelop/kdevcore.h>
#include <kdevelop/kdevproject.h>
#include <kdevelop/kdevpartcontroller.h>
#include <kdevelop/kdevmainwindow.h>

#include "todolist_part.h"
#include "todolist_widget.h"

/**
 * This class is used for the items in the list view. It reimplements compare,
 * to allow item be sorted for file name and then for line number
 *
 * @author Andrea Bergia <andreabergia@yahoo.it>
*/
class TodoListItem : public KListViewItem {
public:
    TodoListItem( QListView* parent, const QString& url, const QString& line,
        const QString& comment )
        : KListViewItem( parent, url, line, comment ) { }

    /**
     * Compare two TodoListItem. Used to sort them in a nice way.
    */
    int compare( QListViewItem* item, int column, bool ascending ) const 
    {
        if ( column == 1 ) {
            int a = text( 1 ).toInt();
            int b = item->text( 1 ).toInt();
            if ( a == b )
                return 0;
            if ( ascending )
                return ( a > b ? 1 : -1 );
            else
                return ( a > b ? -1 : 1 );
        }
        return KListViewItem::compare( item, column, ascending );
    }
};


TodoListWidget::TodoListWidget(TodoListPart *part)
    : QWidget( 0, "todolist widget" ),
    m_part( part )
{
    // Create the list child widget
    m_list = new KListView( this, "todolist listview" );
    m_list->addColumn( i18n( "File name" ));
    m_list->addColumn( i18n( "Line" ));
    m_list->addColumn( i18n( "Comment" ));
    
    connect( m_list, SIGNAL( executed( QListViewItem* ) ), 
        SLOT( open( QListViewItem* ) ));
    
    // Place it in a layout
    QHBoxLayout *layout = new QHBoxLayout( this, 0, KDialog::spacingHint() );
    layout->addWidget( m_list );
    
    // Rescan a saved file
    connect( m_part->partController(), SIGNAL( savedFile( const QString& ) ),
        SLOT( rescan( const QString& ) ) );
}


TodoListWidget::~TodoListWidget()
{
}


void TodoListWidget::projectOpened()
{
    // Rescan all
    scan();
}


void TodoListWidget::projectClosed()
{
    // Nothing to show
    m_list->clear();
}


void TodoListWidget::scan()
{
    // Get the list of files to scan
    KDevProject *proj = m_part->project();
    QStringList files = proj->allFiles();
    
    // Scan every of them
    QString param;    
    if ( !files.empty() )
        for ( QStringList::iterator it = files.begin(); it != files.end(); ++it )
            scan( KURL( proj->projectDirectory() + '/' + *it ) );
}


void TodoListWidget::rescan( const QString& file )
{
    // Rescan only the changed file
    scan( KURL( file ) );
}


void TodoListWidget::scan( const KURL& url )
{
    kdDebug() << "TodoList: scanning file " << url << endl;
    
    // Remove all items regarding this file
    QValueList< QListViewItem* > toRemove;
    for ( QListViewItem* item = m_list->firstChild(); 
        item; item = item->nextSibling() ) {
        if ( item->text(0) == url.url() ) {
            m_list->removeItem( item );
            toRemove.push_back( item );
        }
    }
    
    // Really delete the pointers
    for ( QValueList< QListViewItem*>::iterator it = toRemove.begin();
        it != toRemove.end(); ++it )
        delete (*it);
    
    // Get the file in a local path
    QString local;
    if (url.isLocalFile())
        local = url.path();
    else {
        if (! KIO::NetAccess::download( url, local ) ) {
            KNotifyClient::event( "Could not download file",
                i18n ( "Could not download file" ) );
            return;
        }
    }
    
    // Ok. Now search for TODO or @todo marks in this file
    QFile file( local );
    if ( file.open( IO_ReadOnly ) ) {
        QTextStream stream( &file );
        long nline = 0, i;
        while ( ! stream.eof() ) {
            ++ nline;
            QString line = stream.readLine();
            
            // Search for TODO & @todo
            i = line.find( "TODO" );
            if ( i == -1 )
                i = line.find( "@todo" );
                
            if ( i != -1 ) {
                // Found. Insert a new item
                new TodoListItem( m_list, 
                    url.url(), QString::number( nline ),
                    line.stripWhiteSpace() );
            }
        }
        file.close();
    }
    
    // Sort the list
    m_list->setSorting( 0 );
}


void TodoListWidget::open( QListViewItem* item )
{
    // Get URL & line number
    KURL url = item->text( 0 );
    int line = item->text( 1 ).toInt();
    
    // Show it
    kdDebug() << "TodoList: opening " << url << " at line " << line << endl;    
    m_part->partController()->editDocument( url, line );
    m_part->mainWindow()->lowerView( this );
}


#include "todolist_widget.moc"
